// src/config/plugins/index.ts
var isPluginConfig = (plugin) => plugin !== void 0 && (typeof plugin === "string" || "name" in plugin && plugin.name !== void 0 && typeof plugin.name === "string");

// src/validation/index.ts
var ValidationErrors = /* @__PURE__ */ ((ValidationErrors2) => {
  ValidationErrors2["NOT_A_NONEMPTY_STRING"] = "NOT_A_NONEMPTY_STRING";
  ValidationErrors2["NOT_A_POSITIVE_NUMBER"] = "NOT_A_POSITIVE_NUMBER";
  ValidationErrors2["NOT_AN_UNSIGNED_BIGINT"] = "NOT_AN_UNSIGNED_BIGINT";
  return ValidationErrors2;
})(ValidationErrors || {});
var isNumber = (val) => typeof val === "number" && val === val;
var isBigint = (val) => typeof val === "bigint" && val === val;
var isString = (val) => typeof val === "string";
var assertNotEmptyString = (value) => {
  if (!isString(value) || value.length === 0) {
    throw new ValidationError("NOT_A_NONEMPTY_STRING" /* NOT_A_NONEMPTY_STRING */);
  }
  return value;
};
var assertPositiveNumber = (value) => {
  if (!isNumber(value) || value <= 0) {
    throw new ValidationError("NOT_A_POSITIVE_NUMBER" /* NOT_A_POSITIVE_NUMBER */);
  }
  return value;
};
var assertUnsignedBigInt = (value) => {
  const number = BigInt(value);
  if (number < 0) {
    throw new ValidationError("NOT_AN_UNSIGNED_BIGINT" /* NOT_AN_UNSIGNED_BIGINT */);
  }
  return number;
};

// src/errors/index.ts
var isErrorConstructor = (expect) => {
  return typeof expect === "function" && expect.prototype && // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
  expect.prototype.constructor === expect;
};
var EmmettError = class _EmmettError extends Error {
  errorCode;
  constructor(options) {
    const errorCode = options && typeof options === "object" && "errorCode" in options ? options.errorCode : isNumber(options) ? options : 500;
    const message = options && typeof options === "object" && "message" in options ? options.message : isString(options) ? options : `Error with status code '${errorCode}' ocurred during Emmett processing`;
    super(message);
    this.errorCode = errorCode;
    Object.setPrototypeOf(this, _EmmettError.prototype);
  }
  static mapFrom(error) {
    if (error instanceof _EmmettError) {
      return error;
    }
    return new _EmmettError({
      errorCode: "errorCode" in error && error.errorCode !== void 0 && error.errorCode !== null ? error.errorCode : 500,
      message: error.message ?? "An unknown error occurred"
    });
  }
};
var ConcurrencyError = class _ConcurrencyError extends EmmettError {
  constructor(current, expected, message) {
    super({
      errorCode: 412,
      message: message ?? `Expected version ${expected.toString()} does not match current ${current?.toString()}`
    });
    this.current = current;
    this.expected = expected;
    Object.setPrototypeOf(this, _ConcurrencyError.prototype);
  }
};
var ConcurrencyInMemoryDatabaseError = class _ConcurrencyInMemoryDatabaseError extends EmmettError {
  constructor(message) {
    super({
      errorCode: 412,
      message: message ?? `Expected document state does not match current one!`
    });
    Object.setPrototypeOf(this, _ConcurrencyInMemoryDatabaseError.prototype);
  }
};
var ValidationError = class _ValidationError extends EmmettError {
  constructor(message) {
    super({
      errorCode: 400,
      message: message ?? `Validation Error ocurred during Emmett processing`
    });
    Object.setPrototypeOf(this, _ValidationError.prototype);
  }
};
var IllegalStateError = class _IllegalStateError extends EmmettError {
  constructor(message) {
    super({
      errorCode: 403,
      message: message ?? `Illegal State ocurred during Emmett processing`
    });
    Object.setPrototypeOf(this, _IllegalStateError.prototype);
  }
};
var NotFoundError = class _NotFoundError extends EmmettError {
  constructor(options) {
    super({
      errorCode: 404,
      message: options?.message ?? (options?.id ? options.type ? `${options.type} with ${options.id} was not found during Emmett processing` : `State with ${options.id} was not found during Emmett processing` : options?.type ? `${options.type} was not found during Emmett processing` : "State was not found during Emmett processing")
    });
    Object.setPrototypeOf(this, _NotFoundError.prototype);
  }
};

// src/validation/dates.ts
var formatDateToUtcYYYYMMDD = (date) => {
  const formatter = new Intl.DateTimeFormat("en-CA", {
    timeZone: "UTC",
    year: "numeric",
    month: "2-digit",
    day: "2-digit"
  });
  return formatter.format(date);
};
var isValidYYYYMMDD = (dateString) => {
  const regex = /^\d{4}-\d{2}-\d{2}$/;
  return regex.test(dateString);
};
var parseDateFromUtcYYYYMMDD = (dateString) => {
  const date = /* @__PURE__ */ new Date(dateString + "T00:00:00Z");
  if (!isValidYYYYMMDD(dateString)) {
    throw new ValidationError("Invalid date format, must be yyyy-mm-dd");
  }
  if (isNaN(date.getTime())) {
    throw new ValidationError("Invalid date format");
  }
  return date;
};

export {
  isPluginConfig,
  formatDateToUtcYYYYMMDD,
  isValidYYYYMMDD,
  parseDateFromUtcYYYYMMDD,
  ValidationErrors,
  isNumber,
  isBigint,
  isString,
  assertNotEmptyString,
  assertPositiveNumber,
  assertUnsignedBigInt,
  isErrorConstructor,
  EmmettError,
  ConcurrencyError,
  ConcurrencyInMemoryDatabaseError,
  ValidationError,
  IllegalStateError,
  NotFoundError
};
//# sourceMappingURL=chunk-O2VMBOV4.js.map