#!/usr/bin/env node
"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { newObj[key] = obj[key]; } } } newObj.default = obj; return newObj; } } function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }


var _chunkFMH3M5GBcjs = require('./chunk-FMH3M5GB.cjs');

// src/cli.ts
var _commander = require('commander');

// src/commandLine/config.ts

var _fs = require('fs');
var _process = require('process');
var sampleConfig = (plugins = ["emmett-expressjs"]) => {
  const pluginsNames = plugins.length > 0 ? `[
${plugins.map((p) => `"${p}"`).join(",\n")}  
]` : "[]";
  return `
export default {
  plugins: ${pluginsNames},
};
`;
};
var generateConfigFile = (configPath, collectionNames) => {
  try {
    _fs.writeFileSync.call(void 0, configPath, sampleConfig(collectionNames), "utf8");
    console.log(`Configuration file stored at: ${configPath}`);
  } catch (error) {
    console.error(`Error: Couldn't store config file: ${configPath}!`);
    console.error(error);
    process.exit(1);
  }
};
var configCommand = new (0, _commander.Command)("config").description(
  "Manage Pongo configuration"
);
configCommand.command("sample").description("Generate or print sample configuration").option(
  "-plg, --plugins <name>",
  "Specify the plugin name",
  (value, previous) => {
    return previous.concat([value]);
  },
  []
).option(
  "-f, --file <path>",
  "Path to configuration file with collection list"
).option("-g, --generate", "Generate sample config file").option("-p, --print", "Print sample config file").action((options) => {
  const plugins = options.plugin.length > 0 ? options.plugin : ["@event-driven-io/emmett-expressjs"];
  if (!("print" in options) && !("generate" in options)) {
    console.error(
      "Error: Please provide either:\n--print param to print sample config or\n--generate to generate sample config file"
    );
    _process.exit.call(void 0, 1);
  }
  if ("print" in options) {
    console.log(`${sampleConfig(plugins)}`);
  } else if ("generate" in options) {
    if (!options.file) {
      console.error(
        "Error: You need to provide a config file through a --file"
      );
      _process.exit.call(void 0, 1);
    }
    generateConfigFile(options.file, plugins);
  }
});

// src/commandLine/plugins.ts

var _path = require('path'); var _path2 = _interopRequireDefault(_path);
var PluginsConfigImportError = {
  missingDefaultExport: `Error: Config should contain default export, e.g.

${sampleConfig()}`,
  missingPluginsPropertyExport: `Error: Config should contain default export with plugins array, e.g.

${sampleConfig()}`,
  wrongPluginStructure: `Error: Plugin config should be either string with plugin name or object with plugin name, e.g. { name: 'emmett-expressjs' }`
};
var importPluginsConfig = async (options) => {
  const configPath = _path2.default.join(
    process.cwd(),
    _nullishCoalesce(_optionalChain([options, 'optionalAccess', _ => _.configPath]), () => ( "./dist/emmett.config.js"))
  );
  console.log("IMPORTING" + configPath);
  try {
    const imported = await Promise.resolve().then(() => _interopRequireWildcard(require(configPath)));
    if (!imported.default) {
      return new (0, _chunkFMH3M5GBcjs.EmmettError)(PluginsConfigImportError.missingDefaultExport);
    }
    if (!imported.default.plugins || !Array.isArray(imported.default.plugins)) {
      return new (0, _chunkFMH3M5GBcjs.EmmettError)(
        PluginsConfigImportError.missingPluginsPropertyExport
      );
    }
    if (!imported.default.plugins.every(_chunkFMH3M5GBcjs.isPluginConfig)) {
      return new (0, _chunkFMH3M5GBcjs.EmmettError)(PluginsConfigImportError.wrongPluginStructure);
    }
    return { plugins: imported.default.plugins };
  } catch (error) {
    if (!_optionalChain([options, 'optionalAccess', _2 => _2.configPath])) {
      console.warn("Didn`t find config file: " + configPath, error);
      return { plugins: [] };
    }
    return new (0, _chunkFMH3M5GBcjs.EmmettError)(
      `Error: Couldn't load file:` + error.toString()
    );
  }
};
var loadPlugins = async (options) => {
  try {
    const pluginsConfig = await importPluginsConfig({
      configPath: _optionalChain([options, 'optionalAccess', _3 => _3.configPath])
    });
    if (pluginsConfig instanceof _chunkFMH3M5GBcjs.EmmettError) throw pluginsConfig;
    if (pluginsConfig.plugins.length === 0) {
      console.log(`No extensions specified in config ${_optionalChain([options, 'optionalAccess', _4 => _4.configPath])}.`);
      return [];
    }
    const pluginsToLoad = filterPluginsByType(
      pluginsConfig.plugins,
      _optionalChain([options, 'optionalAccess', _5 => _5.pluginType])
    );
    const pluginsPromises = pluginsToLoad.map(async (pluginConfig) => {
      const importPath = getImportPath(pluginConfig, _optionalChain([options, 'optionalAccess', _6 => _6.pluginType]));
      try {
        const plugin = await Promise.resolve().then(() => _interopRequireWildcard(require(importPath)));
        console.info(`Loaded plugin: ${importPath}`);
        if (!plugin.default) {
          throw new Error(`Plugin: ${importPath} is missing default export`);
        }
        return plugin.default;
      } catch (error) {
        console.error(`Failed to load extension "${importPath}":`, error);
        return void 0;
      }
    });
    return (await Promise.all(pluginsPromises)).filter(
      (plugin) => plugin !== void 0
    );
  } catch (error) {
    console.error(`Failed to load config ${_optionalChain([options, 'optionalAccess', _7 => _7.configPath])}:`, error);
    return [];
  }
};
var registerCliPlugins = async (program2, plugins) => {
  const result = [];
  for (const plugin of plugins) {
    const pluginName = plugin.name;
    if (!("registerCommands" in plugin)) {
      console.warn(`No registerCommands function found in ${pluginName}`);
    }
    await plugin.registerCommands(program2);
    console.log(`Loaded extension: ${plugin.name}`);
    result.push(plugin);
  }
};
var filterPluginsByType = (plugins, pluginType) => plugins.filter(
  (p) => typeof p === "string" || pluginType && (p.register === void 0 || p.register.some((r) => r.pluginType === pluginType))
);
var getImportPath = (pluginConfig, pluginType) => {
  if (typeof pluginConfig === "string") {
    return pluginType ? `${pluginConfig}/${pluginType}` : pluginConfig;
  }
  const pluginSubpath = _nullishCoalesce(_optionalChain([pluginConfig, 'access', _8 => _8.register, 'access', _9 => _9.find, 'call', _10 => _10((r) => pluginType && r.pluginType === pluginType), 'optionalAccess', _11 => _11.path]), () => ( pluginType));
  return pluginSubpath ? `${pluginConfig.name}/${pluginSubpath}` : pluginConfig.name;
};

// src/cli.ts
var program = new (0, _commander.Command)();
program.name("emmett").description("CLI tool for Emmett").option("--config <path>", "Path to the configuration file");
var initCLI = async () => {
  const configIndex = process.argv.indexOf("--config");
  const configPath = configIndex !== -1 && process.argv.length > configIndex + 1 ? process.argv[configIndex + 1] : void 0;
  try {
    const plugins = await loadPlugins({
      pluginType: "cli",
      configPath
    });
    await registerCliPlugins(program, plugins);
    program.parse(process.argv);
  } catch (err) {
    console.error(`Failed to load config from ${configPath}:`, err);
  }
};
initCLI().catch((err) => {
  console.error(`CLI initialization failed:`);
  console.error(err);
});
var cli_default = program;








exports.configCommand = configCommand; exports.default = cli_default; exports.generateConfigFile = generateConfigFile; exports.importPluginsConfig = importPluginsConfig; exports.loadPlugins = loadPlugins; exports.registerCliPlugins = registerCliPlugins; exports.sampleConfig = sampleConfig;
//# sourceMappingURL=cli.cjs.map